(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
	typeof define === 'function' && define.amd ? define(['exports'], factory) :
	(global = global || self, factory(global.Tesfy = {}));
}(this, (function (exports) { 'use strict';

	var commonjsGlobal = typeof globalThis !== 'undefined' ? globalThis : typeof window !== 'undefined' ? window : typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : {};

	function createCommonjsModule(fn, module) {
		return module = { exports: {} }, fn(module, module.exports), module.exports;
	}

	var logic = createCommonjsModule(function (module, exports) {
	(function(root, factory) {
	  {
	    module.exports = factory();
	  }
	}(commonjsGlobal, function() {
	  /* globals console:false */

	  if ( ! Array.isArray) {
	    Array.isArray = function(arg) {
	      return Object.prototype.toString.call(arg) === "[object Array]";
	    };
	  }

	  /**
	   * Return an array that contains no duplicates (original not modified)
	   * @param  {array} array   Original reference array
	   * @return {array}         New array with no duplicates
	   */
	  function arrayUnique(array) {
	    var a = [];
	    for (var i=0, l=array.length; i<l; i++) {
	      if (a.indexOf(array[i]) === -1) {
	        a.push(array[i]);
	      }
	    }
	    return a;
	  }

	  var jsonLogic = {};
	  var operations = {
	    "==": function(a, b) {
	      return a == b;
	    },
	    "===": function(a, b) {
	      return a === b;
	    },
	    "!=": function(a, b) {
	      return a != b;
	    },
	    "!==": function(a, b) {
	      return a !== b;
	    },
	    ">": function(a, b) {
	      return a > b;
	    },
	    ">=": function(a, b) {
	      return a >= b;
	    },
	    "<": function(a, b, c) {
	      return (c === undefined) ? a < b : (a < b) && (b < c);
	    },
	    "<=": function(a, b, c) {
	      return (c === undefined) ? a <= b : (a <= b) && (b <= c);
	    },
	    "!!": function(a) {
	      return jsonLogic.truthy(a);
	    },
	    "!": function(a) {
	      return !jsonLogic.truthy(a);
	    },
	    "%": function(a, b) {
	      return a % b;
	    },
	    "log": function(a) {
	      console.log(a); return a;
	    },
	    "in": function(a, b) {
	      if(!b || typeof b.indexOf === "undefined") return false;
	      return (b.indexOf(a) !== -1);
	    },
	    "cat": function() {
	      return Array.prototype.join.call(arguments, "");
	    },
	    "substr":function(source, start, end) {
	      if(end < 0){
	        // JavaScript doesn't support negative end, this emulates PHP behavior
	        var temp = String(source).substr(start);
	        return temp.substr(0, temp.length + end);
	      }
	      return String(source).substr(start, end);
	    },
	    "+": function() {
	      return Array.prototype.reduce.call(arguments, function(a, b) {
	        return parseFloat(a, 10) + parseFloat(b, 10);
	      }, 0);
	    },
	    "*": function() {
	      return Array.prototype.reduce.call(arguments, function(a, b) {
	        return parseFloat(a, 10) * parseFloat(b, 10);
	      });
	    },
	    "-": function(a, b) {
	      if(b === undefined) {
	        return -a;
	      }else {
	        return a - b;
	      }
	    },
	    "/": function(a, b) {
	      return a / b;
	    },
	    "min": function() {
	      return Math.min.apply(this, arguments);
	    },
	    "max": function() {
	      return Math.max.apply(this, arguments);
	    },
	    "merge": function() {
	      return Array.prototype.reduce.call(arguments, function(a, b) {
	        return a.concat(b);
	      }, []);
	    },
	    "var": function(a, b) {
	      var not_found = (b === undefined) ? null : b;
	      var data = this;
	      if(typeof a === "undefined" || a==="" || a===null) {
	        return data;
	      }
	      var sub_props = String(a).split(".");
	      for(var i = 0; i < sub_props.length; i++) {
	        if(data === null) {
	          return not_found;
	        }
	        // Descending into data
	        data = data[sub_props[i]];
	        if(data === undefined) {
	          return not_found;
	        }
	      }
	      return data;
	    },
	    "missing": function() {
	      /*
	      Missing can receive many keys as many arguments, like {"missing:[1,2]}
	      Missing can also receive *one* argument that is an array of keys,
	      which typically happens if it's actually acting on the output of another command
	      (like 'if' or 'merge')
	      */

	      var missing = [];
	      var keys = Array.isArray(arguments[0]) ? arguments[0] : arguments;

	      for(var i = 0; i < keys.length; i++) {
	        var key = keys[i];
	        var value = jsonLogic.apply({"var": key}, this);
	        if(value === null || value === "") {
	          missing.push(key);
	        }
	      }

	      return missing;
	    },
	    "missing_some": function(need_count, options) {
	      // missing_some takes two arguments, how many (minimum) items must be present, and an array of keys (just like 'missing') to check for presence.
	      var are_missing = jsonLogic.apply({"missing": options}, this);

	      if(options.length - are_missing.length >= need_count) {
	        return [];
	      }else {
	        return are_missing;
	      }
	    },
	    "method": function(obj, method, args) {
	      return obj[method].apply(obj, args);
	    },

	  };

	  jsonLogic.is_logic = function(logic) {
	    return (
	      typeof logic === "object" && // An object
	      logic !== null && // but not null
	      ! Array.isArray(logic) && // and not an array
	      Object.keys(logic).length === 1 // with exactly one key
	    );
	  };

	  /*
	  This helper will defer to the JsonLogic spec as a tie-breaker when different language interpreters define different behavior for the truthiness of primitives.  E.g., PHP considers empty arrays to be falsy, but Javascript considers them to be truthy. JsonLogic, as an ecosystem, needs one consistent answer.

	  Spec and rationale here: http://jsonlogic.com/truthy
	  */
	  jsonLogic.truthy = function(value) {
	    if(Array.isArray(value) && value.length === 0) {
	      return false;
	    }
	    return !! value;
	  };


	  jsonLogic.get_operator = function(logic) {
	    return Object.keys(logic)[0];
	  };

	  jsonLogic.get_values = function(logic) {
	    return logic[jsonLogic.get_operator(logic)];
	  };

	  jsonLogic.apply = function(logic, data) {
	    // Does this array contain logic? Only one way to find out.
	    if(Array.isArray(logic)) {
	      return logic.map(function(l) {
	        return jsonLogic.apply(l, data);
	      });
	    }
	    // You've recursed to a primitive, stop!
	    if( ! jsonLogic.is_logic(logic) ) {
	      return logic;
	    }

	    data = data || {};

	    var op = jsonLogic.get_operator(logic);
	    var values = logic[op];
	    var i;
	    var current;
	    var scopedLogic, scopedData, filtered, initial;

	    // easy syntax for unary operators, like {"var" : "x"} instead of strict {"var" : ["x"]}
	    if( ! Array.isArray(values)) {
	      values = [values];
	    }

	    // 'if', 'and', and 'or' violate the normal rule of depth-first calculating consequents, let each manage recursion as needed.
	    if(op === "if" || op == "?:") {
	      /* 'if' should be called with a odd number of parameters, 3 or greater
	      This works on the pattern:
	      if( 0 ){ 1 }else{ 2 };
	      if( 0 ){ 1 }else if( 2 ){ 3 }else{ 4 };
	      if( 0 ){ 1 }else if( 2 ){ 3 }else if( 4 ){ 5 }else{ 6 };

	      The implementation is:
	      For pairs of values (0,1 then 2,3 then 4,5 etc)
	      If the first evaluates truthy, evaluate and return the second
	      If the first evaluates falsy, jump to the next pair (e.g, 0,1 to 2,3)
	      given one parameter, evaluate and return it. (it's an Else and all the If/ElseIf were false)
	      given 0 parameters, return NULL (not great practice, but there was no Else)
	      */
	      for(i = 0; i < values.length - 1; i += 2) {
	        if( jsonLogic.truthy( jsonLogic.apply(values[i], data) ) ) {
	          return jsonLogic.apply(values[i+1], data);
	        }
	      }
	      if(values.length === i+1) return jsonLogic.apply(values[i], data);
	      return null;
	    }else if(op === "and") { // Return first falsy, or last
	      for(i=0; i < values.length; i+=1) {
	        current = jsonLogic.apply(values[i], data);
	        if( ! jsonLogic.truthy(current)) {
	          return current;
	        }
	      }
	      return current; // Last
	    }else if(op === "or") {// Return first truthy, or last
	      for(i=0; i < values.length; i+=1) {
	        current = jsonLogic.apply(values[i], data);
	        if( jsonLogic.truthy(current) ) {
	          return current;
	        }
	      }
	      return current; // Last




	    }else if(op === 'filter'){
	      scopedData = jsonLogic.apply(values[0], data);
	      scopedLogic = values[1];

	      if ( ! Array.isArray(scopedData)) {
	          return [];
	      }
	      // Return only the elements from the array in the first argument,
	      // that return truthy when passed to the logic in the second argument.
	      // For parity with JavaScript, reindex the returned array
	      return scopedData.filter(function(datum){
	          return jsonLogic.truthy( jsonLogic.apply(scopedLogic, datum));
	      });
	  }else if(op === 'map'){
	      scopedData = jsonLogic.apply(values[0], data);
	      scopedLogic = values[1];

	      if ( ! Array.isArray(scopedData)) {
	          return [];
	      }

	      return scopedData.map(function(datum){
	          return jsonLogic.apply(scopedLogic, datum);
	      });

	  }else if(op === 'reduce'){
	      scopedData = jsonLogic.apply(values[0], data);
	      scopedLogic = values[1];
	      initial = typeof values[2] !== 'undefined' ? values[2] : null;

	      if ( ! Array.isArray(scopedData)) {
	          return initial;
	      }

	      return scopedData.reduce(
	          function(accumulator, current){
	              return jsonLogic.apply(
	                  scopedLogic,
	                  {'current':current, 'accumulator':accumulator}
	              );
	          },
	          initial
	      );

	    }else if(op === "all") {
	      scopedData = jsonLogic.apply(values[0], data);
	      scopedLogic = values[1];
	      // All of an empty set is false. Note, some and none have correct fallback after the for loop
	      if( ! scopedData.length) {
	        return false;
	      }
	      for(i=0; i < scopedData.length; i+=1) {
	        if( ! jsonLogic.truthy( jsonLogic.apply(scopedLogic, scopedData[i]) )) {
	          return false; // First falsy, short circuit
	        }
	      }
	      return true; // All were truthy
	    }else if(op === "none") {
	      filtered = jsonLogic.apply({'filter' : values}, data);
	      return filtered.length === 0;

	    }else if(op === "some") {
	      filtered = jsonLogic.apply({'filter' : values}, data);
	      return filtered.length > 0;
	    }

	    // Everyone else gets immediate depth-first recursion
	    values = values.map(function(val) {
	      return jsonLogic.apply(val, data);
	    });


	    // The operation is called with "data" bound to its "this" and "values" passed as arguments.
	    // Structured commands like % or > can name formal arguments while flexible commands (like missing or merge) can operate on the pseudo-array arguments
	    // https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Functions/arguments
	    if(typeof operations[op] === "function") {
	      return operations[op].apply(data, values);
	    }else if(op.indexOf(".") > 0) { // Contains a dot, and not in the 0th position
	      var sub_ops = String(op).split(".");
	      var operation = operations;
	      for(i = 0; i < sub_ops.length; i++) {
	        // Descending into operations
	        operation = operation[sub_ops[i]];
	        if(operation === undefined) {
	          throw new Error("Unrecognized operation " + op +
	          " (failed at " + sub_ops.slice(0, i+1).join(".") + ")");
	        }
	      }

	      return operation.apply(data, values);
	    }

	    throw new Error("Unrecognized operation " + op );
	  };

	  jsonLogic.uses_data = function(logic) {
	    var collection = [];

	    if( jsonLogic.is_logic(logic) ) {
	      var op = jsonLogic.get_operator(logic);
	      var values = logic[op];

	      if( ! Array.isArray(values)) {
	        values = [values];
	      }

	      if(op === "var") {
	        // This doesn't cover the case where the arg to var is itself a rule.
	        collection.push(values[0]);
	      }else {
	        // Recursion!
	        values.map(function(val) {
	          collection.push.apply(collection, jsonLogic.uses_data(val) );
	        });
	      }
	    }

	    return arrayUnique(collection);
	  };

	  jsonLogic.add_operation = function(name, code) {
	    operations[name] = code;
	  };

	  jsonLogic.rm_operation = function(name) {
	    delete operations[name];
	  };

	  jsonLogic.rule_like = function(rule, pattern) {
	    // console.log("Is ". JSON.stringify(rule) . " like " . JSON.stringify(pattern) . "?");
	    if(pattern === rule) {
	      return true;
	    } // TODO : Deep object equivalency?
	    if(pattern === "@") {
	      return true;
	    } // Wildcard!
	    if(pattern === "number") {
	      return (typeof rule === "number");
	    }
	    if(pattern === "string") {
	      return (typeof rule === "string");
	    }
	    if(pattern === "array") {
	      // !logic test might be superfluous in JavaScript
	      return Array.isArray(rule) && ! jsonLogic.is_logic(rule);
	    }

	    if(jsonLogic.is_logic(pattern)) {
	      if(jsonLogic.is_logic(rule)) {
	        var pattern_op = jsonLogic.get_operator(pattern);
	        var rule_op = jsonLogic.get_operator(rule);

	        if(pattern_op === "@" || pattern_op === rule_op) {
	        // echo "\nOperators match, go deeper\n";
	          return jsonLogic.rule_like(
	            jsonLogic.get_values(rule, false),
	            jsonLogic.get_values(pattern, false)
	          );
	        }
	      }
	      return false; // pattern is logic, rule isn't, can't be eq
	    }

	    if(Array.isArray(pattern)) {
	      if(Array.isArray(rule)) {
	        if(pattern.length !== rule.length) {
	          return false;
	        }
	        /*
	          Note, array order MATTERS, because we're using this array test logic to consider arguments, where order can matter. (e.g., + is commutative, but '-' or 'if' or 'var' are NOT)
	        */
	        for(var i = 0; i < pattern.length; i += 1) {
	          // If any fail, we fail
	          if( ! jsonLogic.rule_like(rule[i], pattern[i])) {
	            return false;
	          }
	        }
	        return true; // If they *all* passed, we pass
	      }else {
	        return false; // Pattern is array, rule isn't
	      }
	    }

	    // Not logic, not array, not a === match for rule.
	    return false;
	  };

	  return jsonLogic;
	}));
	});

	var jsonLogic = {
		__proto__: null,
		'default': logic,
		__moduleExports: logic
	};

	// @ts-ignore

	var AudienceEvaluator = /*#__PURE__*/function () {
	  function AudienceEvaluator() {}

	  var _proto = AudienceEvaluator.prototype;

	  _proto.evaluate = function evaluate(audience, attributes) {
	    if (audience === void 0) {
	      audience = {};
	    }

	    if (attributes === void 0) {
	      attributes = {};
	    }

	    return undefined(audience, attributes);
	  };

	  return AudienceEvaluator;
	}();

	var murmurhash = createCommonjsModule(function (module) {
	(function(){

	  /**
	   * JS Implementation of MurmurHash2
	   *
	   * @author <a href="mailto:gary.court@gmail.com">Gary Court</a>
	   * @see http://github.com/garycourt/murmurhash-js
	   * @author <a href="mailto:aappleby@gmail.com">Austin Appleby</a>
	   * @see http://sites.google.com/site/murmurhash/
	   *
	   * @param {string} str ASCII only
	   * @param {number} seed Positive integer only
	   * @return {number} 32-bit positive integer hash
	   */
	  function MurmurHashV2(str, seed) {
	    var
	      l = str.length,
	      h = seed ^ l,
	      i = 0,
	      k;

	    while (l >= 4) {
	      k =
	        ((str.charCodeAt(i) & 0xff)) |
	        ((str.charCodeAt(++i) & 0xff) << 8) |
	        ((str.charCodeAt(++i) & 0xff) << 16) |
	        ((str.charCodeAt(++i) & 0xff) << 24);

	      k = (((k & 0xffff) * 0x5bd1e995) + ((((k >>> 16) * 0x5bd1e995) & 0xffff) << 16));
	      k ^= k >>> 24;
	      k = (((k & 0xffff) * 0x5bd1e995) + ((((k >>> 16) * 0x5bd1e995) & 0xffff) << 16));

	    h = (((h & 0xffff) * 0x5bd1e995) + ((((h >>> 16) * 0x5bd1e995) & 0xffff) << 16)) ^ k;

	      l -= 4;
	      ++i;
	    }

	    switch (l) {
	    case 3: h ^= (str.charCodeAt(i + 2) & 0xff) << 16;
	    case 2: h ^= (str.charCodeAt(i + 1) & 0xff) << 8;
	    case 1: h ^= (str.charCodeAt(i) & 0xff);
	            h = (((h & 0xffff) * 0x5bd1e995) + ((((h >>> 16) * 0x5bd1e995) & 0xffff) << 16));
	    }

	    h ^= h >>> 13;
	    h = (((h & 0xffff) * 0x5bd1e995) + ((((h >>> 16) * 0x5bd1e995) & 0xffff) << 16));
	    h ^= h >>> 15;

	    return h >>> 0;
	  }
	  /**
	   * JS Implementation of MurmurHash3 (r136) (as of May 20, 2011)
	   *
	   * @author <a href="mailto:gary.court@gmail.com">Gary Court</a>
	   * @see http://github.com/garycourt/murmurhash-js
	   * @author <a href="mailto:aappleby@gmail.com">Austin Appleby</a>
	   * @see http://sites.google.com/site/murmurhash/
	   *
	   * @param {string} key ASCII only
	   * @param {number} seed Positive integer only
	   * @return {number} 32-bit positive integer hash
	   */
	  function MurmurHashV3(key, seed) {
	    var remainder, bytes, h1, h1b, c1, c2, k1, i;

	    remainder = key.length & 3; // key.length % 4
	    bytes = key.length - remainder;
	    h1 = seed;
	    c1 = 0xcc9e2d51;
	    c2 = 0x1b873593;
	    i = 0;

	    while (i < bytes) {
	        k1 =
	          ((key.charCodeAt(i) & 0xff)) |
	          ((key.charCodeAt(++i) & 0xff) << 8) |
	          ((key.charCodeAt(++i) & 0xff) << 16) |
	          ((key.charCodeAt(++i) & 0xff) << 24);
	      ++i;

	      k1 = ((((k1 & 0xffff) * c1) + ((((k1 >>> 16) * c1) & 0xffff) << 16))) & 0xffffffff;
	      k1 = (k1 << 15) | (k1 >>> 17);
	      k1 = ((((k1 & 0xffff) * c2) + ((((k1 >>> 16) * c2) & 0xffff) << 16))) & 0xffffffff;

	      h1 ^= k1;
	          h1 = (h1 << 13) | (h1 >>> 19);
	      h1b = ((((h1 & 0xffff) * 5) + ((((h1 >>> 16) * 5) & 0xffff) << 16))) & 0xffffffff;
	      h1 = (((h1b & 0xffff) + 0x6b64) + ((((h1b >>> 16) + 0xe654) & 0xffff) << 16));
	    }

	    k1 = 0;

	    switch (remainder) {
	      case 3: k1 ^= (key.charCodeAt(i + 2) & 0xff) << 16;
	      case 2: k1 ^= (key.charCodeAt(i + 1) & 0xff) << 8;
	      case 1: k1 ^= (key.charCodeAt(i) & 0xff);

	      k1 = (((k1 & 0xffff) * c1) + ((((k1 >>> 16) * c1) & 0xffff) << 16)) & 0xffffffff;
	      k1 = (k1 << 15) | (k1 >>> 17);
	      k1 = (((k1 & 0xffff) * c2) + ((((k1 >>> 16) * c2) & 0xffff) << 16)) & 0xffffffff;
	      h1 ^= k1;
	    }

	    h1 ^= key.length;

	    h1 ^= h1 >>> 16;
	    h1 = (((h1 & 0xffff) * 0x85ebca6b) + ((((h1 >>> 16) * 0x85ebca6b) & 0xffff) << 16)) & 0xffffffff;
	    h1 ^= h1 >>> 13;
	    h1 = ((((h1 & 0xffff) * 0xc2b2ae35) + ((((h1 >>> 16) * 0xc2b2ae35) & 0xffff) << 16))) & 0xffffffff;
	    h1 ^= h1 >>> 16;

	    return h1 >>> 0;
	  }

	  var murmur = MurmurHashV3;
	  murmur.v2 = MurmurHashV2;
	  murmur.v3 = MurmurHashV3;

	  {
	    module.exports = murmur;
	  }
	}());
	});

	var murmurhash$1 = {
		__proto__: null,
		'default': murmurhash,
		__moduleExports: murmurhash
	};

	// @ts-ignore

	var Bucketer = /*#__PURE__*/function () {
	  function Bucketer(maxBuckets) {
	    this.maxBuckets = maxBuckets;
	  }

	  var _proto = Bucketer.prototype;

	  _proto.computeBucketId = function computeBucketId(id) {
	    // @ts-ignore
	    var hashValue = undefined(id, Bucketer.HASH_SEED);
	    var ratio = hashValue / Bucketer.MAX_HASH_VALUE;
	    return Math.floor(ratio * this.maxBuckets);
	  };

	  _proto.bucket = function bucket(key, allocations) {
	    var bucketId = this.computeBucketId(key);
	    var allocation = allocations.find(function (allocation) {
	      return bucketId < allocation.rangeEnd;
	    });

	    if (allocation) {
	      return allocation.id;
	    }

	    return null;
	  };

	  return Bucketer;
	}();

	Bucketer.HASH_SEED = 1;
	Bucketer.MAX_HASH_VALUE = /*#__PURE__*/Math.pow(2, 32) - 1;

	var Config = /*#__PURE__*/function () {
	  function Config(datafile, maxBuckets) {
	    this.datafile = datafile;
	    this.maxBuckets = maxBuckets;
	  }

	  var _proto = Config.prototype;

	  _proto.computeRangeEnd = function computeRangeEnd(percentage) {
	    return Math.floor(percentage * this.maxBuckets / 100);
	  };

	  _proto.getExperiments = function getExperiments() {
	    var _this$datafile$experi = this.datafile.experiments,
	        experiments = _this$datafile$experi === void 0 ? {} : _this$datafile$experi;
	    return experiments;
	  };

	  _proto.getExperiment = function getExperiment(id) {
	    var experiments = this.getExperiments();
	    return experiments[id];
	  };

	  _proto.getFeatures = function getFeatures() {
	    var _this$datafile$featur = this.datafile.features,
	        features = _this$datafile$featur === void 0 ? {} : _this$datafile$featur;
	    return features;
	  };

	  _proto.getFeature = function getFeature(id) {
	    var features = this.getFeatures();
	    return features[id];
	  };

	  _proto.getFeatureAllocation = function getFeatureAllocation(id) {
	    var feature = this.getFeature(id);

	    if (!feature) {
	      return;
	    }

	    var rangeEnd = this.computeRangeEnd(feature.percentage);
	    return {
	      id: id,
	      rangeEnd: rangeEnd
	    };
	  };

	  _proto.getExperimentAllocation = function getExperimentAllocation(id) {
	    var experiment = this.getExperiment(id);

	    if (!experiment) {
	      return;
	    }

	    var rangeEnd = this.computeRangeEnd(experiment.percentage);
	    return {
	      id: id,
	      rangeEnd: rangeEnd
	    };
	  };

	  _proto.getExperimentAllocations = function getExperimentAllocations(id) {
	    var _this = this;

	    var experiment = this.getExperiment(id);
	    var acc = 0;
	    return experiment.variations.map(function (_ref) {
	      var id = _ref.id,
	          percentage = _ref.percentage;
	      acc += percentage;

	      var rangeEnd = _this.computeRangeEnd(acc);

	      return {
	        id: id,
	        rangeEnd: rangeEnd
	      };
	    });
	  };

	  return Config;
	}();

	function _extends() {
	  _extends = Object.assign || function (target) {
	    for (var i = 1; i < arguments.length; i++) {
	      var source = arguments[i];

	      for (var key in source) {
	        if (Object.prototype.hasOwnProperty.call(source, key)) {
	          target[key] = source[key];
	        }
	      }
	    }

	    return target;
	  };

	  return _extends.apply(this, arguments);
	}

	var Engine = /*#__PURE__*/function () {
	  function Engine(_ref) {
	    var datafile = _ref.datafile,
	        userId = _ref.userId,
	        attributes = _ref.attributes,
	        storage = _ref.storage;
	    this.cache = {};
	    this.config = new Config(datafile, Engine.TOTAL_BUCKETS);
	    this.bucketer = new Bucketer(Engine.TOTAL_BUCKETS);
	    this.evaluator = new AudienceEvaluator();
	    this.storage = storage;
	    this.userId = userId;
	    this.attributes = attributes;
	  }

	  var _proto = Engine.prototype;

	  _proto.computeKey = function computeKey(id, userId, salt) {
	    if (userId === void 0) {
	      userId = '';
	    }

	    if (salt === void 0) {
	      salt = '';
	    }

	    return (userId || this.userId || '').concat(id).concat(salt);
	  };

	  _proto.getForcedVariation = function getForcedVariation(experimentId) {
	    return this.cache[experimentId];
	  };

	  _proto.getUserId = function getUserId() {
	    return this.userId;
	  };

	  _proto.getAttributes = function getAttributes() {
	    return this.attributes;
	  };

	  _proto.setUserId = function setUserId(userId) {
	    this.userId = userId;
	  };

	  _proto.setAttributes = function setAttributes(attributes) {
	    if (attributes === void 0) {
	      attributes = {};
	    }

	    this.attributes = attributes;
	  };

	  _proto.setForcedVariation = function setForcedVariation(experimentId, variationId) {
	    this.cache[experimentId] = variationId;
	  };

	  _proto.isFeatureEnabled = function isFeatureEnabled(featureId, userId, attributes) {
	    var key = this.computeKey(featureId, userId);
	    var feature = this.config.getFeature(featureId);

	    if (!feature) {
	      return null;
	    }

	    var audience = feature.audience;
	    var allocation = this.config.getFeatureAllocation(featureId);

	    if (!allocation || !this.evaluator.evaluate(audience, attributes || this.attributes)) {
	      return null;
	    }

	    return !!this.bucketer.bucket(key, [allocation]);
	  };

	  _proto.getFeature = function getFeature(featureId) {
	    return this.config.getFeature(featureId);
	  };

	  _proto.getEnabledFeatures = function getEnabledFeatures(userId, attributes) {
	    var _this = this;

	    var features = this.config.getFeatures();
	    return Object.keys(features).reduce(function (features, featureId) {
	      var _extends2;

	      return _extends({}, features, (_extends2 = {}, _extends2[featureId] = _this.isFeatureEnabled(featureId, userId, attributes), _extends2));
	    }, {});
	  };

	  _proto.getVariationId = function getVariationId(experimentId, userId, attributes) {
	    var _this$storage, _this$storage2;

	    var variationId = this.getForcedVariation(experimentId) || ((_this$storage = this.storage) === null || _this$storage === void 0 ? void 0 : _this$storage.get(experimentId));

	    if (variationId) {
	      return variationId;
	    }

	    var experiment = this.config.getExperiment(experimentId);

	    if (!experiment) {
	      return null;
	    }

	    var audience = experiment.audience;

	    if (!this.evaluator.evaluate(audience, attributes || this.attributes)) {
	      return null;
	    }

	    var key = this.computeKey(experimentId, userId, Engine.TRAFFIC_ALLOCATION_SALT);
	    var allocation = this.config.getExperimentAllocation(experimentId);

	    if (!allocation || !this.bucketer.bucket(key, [allocation])) {
	      return null;
	    }

	    key = this.computeKey(experimentId, userId);
	    var allocations = this.config.getExperimentAllocations(experimentId);
	    variationId = this.bucketer.bucket(key, allocations);
	    (_this$storage2 = this.storage) === null || _this$storage2 === void 0 ? void 0 : _this$storage2.store(experimentId, variationId);
	    return variationId;
	  };

	  _proto.getVariationIds = function getVariationIds(userId, attributes) {
	    var _this2 = this;

	    var experiments = this.config.getExperiments();
	    return Object.keys(experiments).reduce(function (experiments, experimentId) {
	      var _extends3;

	      return _extends({}, experiments, (_extends3 = {}, _extends3[experimentId] = _this2.getVariationId(experimentId, userId, attributes), _extends3));
	    }, {});
	  };

	  return Engine;
	}();

	Engine.TOTAL_BUCKETS = 10000;
	Engine.TRAFFIC_ALLOCATION_SALT = 'tas';

	exports.AudienceEvaluator = AudienceEvaluator;
	exports.Bucketer = Bucketer;
	exports.Config = Config;
	exports.Engine = Engine;

	Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=tesfy.umd.development.js.map
